#define NO_ADDR_DEFINES
#define NO_BIT_DEFINES
#include "pic14regs.h"
#include "onewire.h"
//#include <string.h>	// memcpy, memset

void memcpy(byte*dst,const byte*src,byte len) {
 do *dst++=*src++; while(--len);
}
void memset(byte*dst,byte data,byte len) {
 do *dst++=data; while(--len);
}

#define owL	(LATB &=~0x20)	// Low
#define owH	(LATB |= 0x20)	// High
#define owT	(TRISB|= 0x20)	// tristate
#define owD	(TRISB&=~0x20)	// driving
#define owR	(PORTB & 0x20)	// read line value
#define di()	(INTCON&=~_GIE)
#define ei()	(INTCON|= _GIE)

static void delay(unsigned t) {
 __asm__(
"	incf	0x7F,f	\n"
"l1:	decfsz	WREG,f	\n"
"	 bra	l1	\n"
"	decfsz	0x7F,f	\n"
"	 bra	l1	\n"
 );
}

// Bei 12 MHz CPU-Frequenz
#define delayMicroseconds(us) delay(us*4)

rom_t owRomNo;
byte owLastDiscrepancy;
byte owLastFamilyDiscrepancy;
bool owLastDeviceFlag;

// Perform the onewire reset function.  We will wait up to 250 µs for
// the bus to come high, if it doesn't then it is broken or shorted
// and we return false;
bool owReset() {
 byte retries;
 bool r;
 owT;
 owH;
 retries = 125;
 do {	// wait until the wire is high... just in case
  if (--retries == 0) return false;
  delayMicroseconds(2);
 } while (!owR);
 owL; owD;	// drive output low
 delayMicroseconds(480);
 di();
 owT; owH;	// allow it to float
 delayMicroseconds(70);
 r = !owR;
 ei();
 delayMicroseconds(410);
 return r;
}

void owWriteBit(byte v) {
 di();
 owD;owL;	// drive output low
 if (v) {
  delayMicroseconds(10);
  owH;	// drive output high
  ei();
  delayMicroseconds(55);
 }else{
  delayMicroseconds(65);
  owH;	// drive output high
  ei();
  delayMicroseconds(5);
 }
}

// Write a byte. The writing code uses the active drivers to raise the pin high
void owWriteByte(byte v) {
 byte i=8;
 do{
  owWriteBit(v&1);	// start LSB first
  v>>=1;
 }while(--i);
}

void owWrite(const byte*buf, byte count) {
 if (count) do{
  owWriteByte(*buf++);
 }while(--count);
}

bool owReadBit() {
 bool r;
 di();
 owD;owL;
 delayMicroseconds(3);
 owT;owH;	// let pin float, pull up will raise
 delayMicroseconds(10);
 r = !!owR;
 ei();
 delayMicroseconds(53);
 return r;
}

byte owReadByte() {
 byte r=0;
 byte m=1;
 do if (owReadBit()) r|=m;
 while(m<<=1);
 return (byte)r;
}

void owRead(byte*buf, byte count) {
 if (count) do{
  *buf++=owReadByte();
 }while(--count);
}

// You need to use this function to start a search again from the beginning.
// You do not need to do it for the first search, though you could.
void owsReset() {
  // reset the search state
 owLastDiscrepancy = 0;
 owLastDeviceFlag = false;
 owLastFamilyDiscrepancy = 0;
 memset(owRomNo,0,8);
}

#if 0

// Setup the search to find the device type 'family_code' on the next call
// to search(*newAddr) if it is present.
void owsTarget(byte family_code) {
 owsReset();
   // set the search state to find SearchFamily type devices
 owRomNo[0]=family_code;
 owLastDiscrepancy = 64;
}

// Perform a search. If this function returns a '1' then it has
// enumerated the next device and you may retrieve the ROM from the
// OneWire::address variable. If there are no devices, no further
// devices, or something horrible happens in the middle of the
// enumeration then a 0 is returned.  If a new device is found then
// its address is copied to newAddr.  Use OneWire::reset_search() to
// start over.
//
// --- Replaced by the one from the Dallas Semiconductor web site ---
//--------------------------------------------------------------------------
// Perform the 1-Wire Search Algorithm on the 1-Wire bus using the existing
// search state.
// Return TRUE  : device found, ROM number in ROM_NO buffer
//        FALSE : device not found, end of search
//
bool owsSearch(byte newAddr[8]) {
 byte id_bit_number=1;
 byte last_zero=0, rom_byte_number=0;
 bool search_result=false;
 bool id_bit, cmp_id_bit;
 byte rom_byte_mask=1;
 bool search_direction;

// if the last call was not the last one
 if (!owLastDeviceFlag) {
  if (!owReset()) {		// 1-Wire reset okay?
   owLastDiscrepancy = 0;	// No: reset the search
   owLastDeviceFlag = false;
   owLastFamilyDiscrepancy = 0;
   return false;
  }
  owWriteByte(0xF0);		// issue the search command
  do{				// loop to do the search
   id_bit = owReadBit();	// read a bit and its complement
   cmp_id_bit = owReadBit();
	// check for no devices on 1-wire
   if (id_bit && cmp_id_bit) break;
   else{
    if (id_bit!=cmp_id_bit)	// all devices coupled have 0 or 1
      search_direction = id_bit;  // bit write value for search
    else{	// if this discrepancy if before the Last Discrepancy
		// on a previous next then pick the same as last time
     if (id_bit_number < owLastDiscrepancy)
       search_direction = ((owRomNo[rom_byte_number] & rom_byte_mask) > 0);
     else	// if equal to last pick 1, if not then pick 0
       search_direction = id_bit_number == owLastDiscrepancy;
               // if 0 was picked then record its position in LastZero
     if (!search_direction) {
      last_zero = id_bit_number;
                  // check for Last discrepancy in family
      if (last_zero < 9) owLastFamilyDiscrepancy = last_zero;
     }
    }
		// set or clear the bit in the ROM byte rom_byte_number
		// with mask rom_byte_mask
    if (search_direction) owRomNo[rom_byte_number] |= rom_byte_mask;
    else owRomNo[rom_byte_number] &=~rom_byte_mask;
		// serial number search direction write bit
    owWriteBit(search_direction);
		// increment the byte counter id_bit_number
		// and shift the mask rom_byte_mask
    id_bit_number++;
    rom_byte_mask<<=1;
		// if the mask is 0 then go to new SerialNum byte rom_byte_number and reset mask
    if (!rom_byte_mask) {rom_byte_mask=1; rom_byte_number++;}
   }
  }
  while (rom_byte_number < 8);  // loop until through all ROM bytes 0-7
      // if the search was successful then
  if (!(id_bit_number < 65)) {
         // search successful so set LastDiscrepancy,LastDeviceFlag,search_result
   owLastDiscrepancy = last_zero;
         // check for last device
   if (!owLastDiscrepancy) owLastDeviceFlag = true;
   search_result = true;
  }
 }
// if no device found then reset counters so next 'search' will be like a first
 if (!search_result || !owRomNo[0]) {
  owLastDiscrepancy = 0;
  owLastDeviceFlag = false;
  owLastFamilyDiscrepancy = 0;
  search_result = false;
 }
 memcpy(newAddr,owRomNo,8);
 return search_result;
}

// The 1-Wire CRC scheme is described in Maxim Application Note 27:
// "Understanding and Using Cyclic Redundancy Checks with Maxim iButton Products"
byte owCrc8(const byte*addr, byte len) {
 byte crc = 0;
 if (len) do{
  byte b = *addr++;
  byte i=8;
  do{
   byte mix = crc^b;	// "bool" compiliert schlecht (XC8 freie Version)
   crc >>= 1;
   if (mix&1) crc ^= 0x8C;
   b>>=1;
  }while(--i);
 }while(--len);
 return crc;
}
#endif